package com.hungraim.ism.service;

import com.hungraim.ism.api.SystemUserFeignService;
import com.hungraim.ism.pojo.entity.system.SystemUser;
import com.hungraim.ism.pojo.vo.system.UserLoginInfoVO;
import com.hungraim.ism.util.Result;
import com.hungraim.ism.util.ResultCode;
import lombok.AllArgsConstructor;
import org.springframework.security.core.userdetails.UserDetails;
import org.springframework.security.core.userdetails.UserDetailsService;
import org.springframework.security.core.userdetails.UsernameNotFoundException;
import org.springframework.stereotype.Service;

import java.util.List;

/**
 * @author hubin
 */
@Service
@AllArgsConstructor
public class UserDetailsServiceImpl implements UserDetailsService {

    private final SystemUserFeignService systemUserFeignService;


    @Override
    public UserDetails loadUserByUsername(String account) throws UsernameNotFoundException {
        //获取用户基本信息
        Result<SystemUser> userRes = systemUserFeignService.loadUserInfoByAccount(account);
        //用户不存在
        if (ResultCode.USER_NOT_EXIST.getCode().equals(userRes.getCode())) {
            throw new UsernameNotFoundException(ResultCode.USER_NOT_EXIST.getMsg());
        }
        SystemUser systemUser = userRes.getData();
        //根据用户id获取用户权限
        Result<List<Long>> roles = systemUserFeignService.loadUserRolesByUserId(systemUser.getId());
        UserLoginInfoVO userLoginInfoVO = new UserLoginInfoVO(systemUser);
        userLoginInfoVO.setAuthorities(roles.getData());

//        Result<SystemMarketer> systemMarketerRes = systemUserFeignService.loadUserDetailByCode(systemUser.getMarketerCode());
//        if (!ResultCode.MARKETER_NOT_EXIST.getCode().equals(systemMarketerRes.getCode())) {
//            SystemMarketer systemMarketer = systemMarketerRes.getData();
//            userLoginInfoVO.setAvatar(systemMarketer.getAvatar());
//        }
        return userLoginInfoVO;
    }
}
